@@ -1,7 +1,7 @@
 package HTML::HTML5::Entities;
 
 use 5.008001;
-use base qw[Exporter];
+use parent qw[Exporter];
 use constant { FALSE => 0, TRUE => 1 };
 use strict;
 use utf8;
@@ -10,7 +10,7 @@ BEGIN {
 	$HTML::HTML5::Entities::AUTHORITY = 'cpan:TOBYINK';
 }
 BEGIN {
-	$HTML::HTML5::Entities::VERSION   = '0.001';
+	$HTML::HTML5::Entities::VERSION   = '0.002';
 }
 
 our (%entity2char, %char2entity, $hex, $default_unsafe_characters, @EXPORT, @EXPORT_OK);
@@ -2256,15 +2256,261 @@ BEGIN
 		"zwj;" => "\x{200d}",
 		"zwnj;" => "\x{200c}",
 		);
-	
-	while (my ($entity, $char) = each %entity2char)
-	{
-		unless (defined $char2entity{$char} and $char2entity{$char} =~ /;$/)
-		{
-			$char2entity{$char} = "&$entity";
-		}
-	}
-	delete $char2entity{"'"};
+
+	%char2entity = (
+		"\x{00a0}"  => '&nbsp;',
+		"\x{00a1}"  => '&iexcl;',
+		"\x{00a2}"  => '&cent;',
+		"\x{00a3}"  => '&pound;',
+		"\x{00a4}"  => '&curren;',
+		"\x{00a5}"  => '&yen;',
+		"\x{00a6}"  => '&brvbar;',
+		"\x{00a7}"  => '&sect;',
+		"\x{00a8}"  => '&uml;',
+		"\x{00a9}"  => '&copy;',
+		"\x{00aa}"  => '&ordf;',
+		"\x{00ab}"  => '&laquo;',
+		"\x{00ac}"  => '&not;',
+		"\x{00ad}"  => '&shy;',
+		"\x{00ae}"  => '&reg;',
+		"\x{00af}"  => '&macr;',
+		"\x{00b0}"  => '&deg;',
+		"\x{00b1}"  => '&plusmn;',
+		"\x{00b2}"  => '&sup2;',
+		"\x{00b3}"  => '&sup3;',
+		"\x{00b4}"  => '&acute;',
+		"\x{00b5}"  => '&micro;',
+		"\x{00b6}"  => '&para;',
+		"\x{00b7}"  => '&middot;',
+		"\x{00b8}"  => '&cedil;',
+		"\x{00b9}"  => '&sup1;',
+		"\x{00ba}"  => '&ordm;',
+		"\x{00bb}"  => '&raquo;',
+		"\x{00bc}"  => '&frac14;',
+		"\x{00bd}"  => '&frac12;',
+		"\x{00be}"  => '&frac34;',
+		"\x{00bf}"  => '&iquest;',
+		"\x{00c0}"  => '&Agrave;',
+		"\x{00c1}"  => '&Aacute;',
+		"\x{00c2}"  => '&Acirc;',
+		"\x{00c3}"  => '&Atilde;',
+		"\x{00c4}"  => '&Auml;',
+		"\x{00c5}"  => '&Aring;',
+		"\x{00c6}"  => '&AElig;',
+		"\x{00c7}"  => '&Ccedil;',
+		"\x{00c8}"  => '&Egrave;',
+		"\x{00c9}"  => '&Eacute;',
+		"\x{00ca}"  => '&Ecirc;',
+		"\x{00cb}"  => '&Euml;',
+		"\x{00cc}"  => '&Igrave;',
+		"\x{00cd}"  => '&Iacute;',
+		"\x{00ce}"  => '&Icirc;',
+		"\x{00cf}"  => '&Iuml;',
+		"\x{00d0}"  => '&ETH;',
+		"\x{00d1}"  => '&Ntilde;',
+		"\x{00d2}"  => '&Ograve;',
+		"\x{00d3}"  => '&Oacute;',
+		"\x{00d4}"  => '&Ocirc;',
+		"\x{00d5}"  => '&Otilde;',
+		"\x{00d6}"  => '&Ouml;',
+		"\x{00d7}"  => '&times;',
+		"\x{00d8}"  => '&Oslash;',
+		"\x{00d9}"  => '&Ugrave;',
+		"\x{00da}"  => '&Uacute;',
+		"\x{00db}"  => '&Ucirc;',
+		"\x{00dc}"  => '&Uuml;',
+		"\x{00dd}"  => '&Yacute;',
+		"\x{00de}"  => '&THORN;',
+		"\x{00df}"  => '&szlig;',
+		"\x{00e0}"  => '&agrave;',
+		"\x{00e1}"  => '&aacute;',
+		"\x{00e2}"  => '&acirc;',
+		"\x{00e3}"  => '&atilde;',
+		"\x{00e4}"  => '&auml;',
+		"\x{00e5}"  => '&aring;',
+		"\x{00e6}"  => '&aelig;',
+		"\x{00e7}"  => '&ccedil;',
+		"\x{00e8}"  => '&egrave;',
+		"\x{00e9}"  => '&eacute;',
+		"\x{00ea}"  => '&ecirc;',
+		"\x{00eb}"  => '&euml;',
+		"\x{00ec}"  => '&igrave;',
+		"\x{00ed}"  => '&iacute;',
+		"\x{00ee}"  => '&icirc;',
+		"\x{00ef}"  => '&iuml;',
+		"\x{00f0}"  => '&eth;',
+		"\x{00f1}"  => '&ntilde;',
+		"\x{00f2}"  => '&ograve;',
+		"\x{00f3}"  => '&oacute;',
+		"\x{00f4}"  => '&ocirc;',
+		"\x{00f5}"  => '&otilde;',
+		"\x{00f6}"  => '&ouml;',
+		"\x{00f7}"  => '&divide;',
+		"\x{00f8}"  => '&oslash;',
+		"\x{00f9}"  => '&ugrave;',
+		"\x{00fa}"  => '&uacute;',
+		"\x{00fb}"  => '&ucirc;',
+		"\x{00fc}"  => '&uuml;',
+		"\x{00fd}"  => '&yacute;',
+		"\x{00fe}"  => '&thorn;',
+		"\x{00ff}"  => '&yuml;',
+		"\x{0026}"  => '&amp;',
+		"\x{0022}"  => '&quot;',
+		"\x{003c}"  => '&lt;',
+		"\x{003e}"  => '&gt;',
+		"\x{0152}"  => '&OElig;',
+		"\x{0153}"  => '&oelig;',
+		"\x{0160}"  => '&Scaron;',
+		"\x{0161}"  => '&scaron;',
+		"\x{0178}"  => '&Yuml;',
+		"\x{02c6}"  => '&circ;',
+		"\x{02dc}"  => '&tilde;',
+		"\x{2002}"  => '&ensp;',
+		"\x{2003}"  => '&emsp;',
+		"\x{2009}"  => '&thinsp;',
+		"\x{200c}"  => '&zwnj;',
+		"\x{200d}"  => '&zwj;',
+		"\x{200e}"  => '&lrm;',
+		"\x{200f}"  => '&rlm;',
+		"\x{2013}"  => '&ndash;',
+		"\x{2014}"  => '&mdash;',
+		"\x{2018}"  => '&lsquo;',
+		"\x{2019}"  => '&rsquo;',
+		"\x{201a}"  => '&sbquo;',
+		"\x{201c}"  => '&ldquo;',
+		"\x{201d}"  => '&rdquo;',
+		"\x{201e}"  => '&bdquo;',
+		"\x{2020}"  => '&dagger;',
+		"\x{2021}"  => '&Dagger;',
+		"\x{2030}"  => '&permil;',
+		"\x{2039}"  => '&lsaquo;',
+		"\x{203a}"  => '&rsaquo;',
+		"\x{20ac}"  => '&euro;',
+		"\x{0192}"  => '&fnof;',
+		"\x{0391}"  => '&Alpha;',
+		"\x{0392}"  => '&Beta;',
+		"\x{0393}"  => '&Gamma;',
+		"\x{0394}"  => '&Delta;',
+		"\x{0395}"  => '&Epsilon;',
+		"\x{0396}"  => '&Zeta;',
+		"\x{0397}"  => '&Eta;',
+		"\x{0398}"  => '&Theta;',
+		"\x{0399}"  => '&Iota;',
+		"\x{039a}"  => '&Kappa;',
+		"\x{039b}"  => '&Lambda;',
+		"\x{039c}"  => '&Mu;',
+		"\x{039d}"  => '&Nu;',
+		"\x{039e}"  => '&Xi;',
+		"\x{039f}"  => '&Omicron;',
+		"\x{03a0}"  => '&Pi;',
+		"\x{03a1}"  => '&Rho;',
+		"\x{03a3}"  => '&Sigma;',
+		"\x{03a4}"  => '&Tau;',
+		"\x{03a5}"  => '&Upsilon;',
+		"\x{03a6}"  => '&Phi;',
+		"\x{03a7}"  => '&Chi;',
+		"\x{03a8}"  => '&Psi;',
+		"\x{03a9}"  => '&Omega;',
+		"\x{03b1}"  => '&alpha;',
+		"\x{03b2}"  => '&beta;',
+		"\x{03b3}"  => '&gamma;',
+		"\x{03b4}"  => '&delta;',
+		"\x{03b5}"  => '&epsilon;',
+		"\x{03b6}"  => '&zeta;',
+		"\x{03b7}"  => '&eta;',
+		"\x{03b8}"  => '&theta;',
+		"\x{03b9}"  => '&iota;',
+		"\x{03ba}"  => '&kappa;',
+		"\x{03bb}"  => '&lambda;',
+		"\x{03bc}"  => '&mu;',
+		"\x{03bd}"  => '&nu;',
+		"\x{03be}"  => '&xi;',
+		"\x{03bf}"  => '&omicron;',
+		"\x{03c0}"  => '&pi;',
+		"\x{03c1}"  => '&rho;',
+		"\x{03c2}"  => '&sigmaf;',
+		"\x{03c3}"  => '&sigma;',
+		"\x{03c4}"  => '&tau;',
+		"\x{03c5}"  => '&upsilon;',
+		"\x{03c6}"  => '&phi;',
+		"\x{03c7}"  => '&chi;',
+		"\x{03c8}"  => '&psi;',
+		"\x{03c9}"  => '&omega;',
+		"\x{03d1}"  => '&thetasym;',
+		"\x{03d2}"  => '&upsih;',
+		"\x{03d6}"  => '&piv;',
+		"\x{2022}"  => '&bull;',
+		"\x{2026}"  => '&hellip;',
+		"\x{2032}"  => '&prime;',
+		"\x{2033}"  => '&Prime;',
+		"\x{203e}"  => '&oline;',
+		"\x{2044}"  => '&frasl;',
+		"\x{2118}"  => '&weierp;',
+		"\x{2111}"  => '&image;',
+		"\x{211c}"  => '&real;',
+		"\x{2122}"  => '&trade;',
+		"\x{2135}"  => '&alefsym;',
+		"\x{2190}"  => '&larr;',
+		"\x{2191}"  => '&uarr;',
+		"\x{2192}"  => '&rarr;',
+		"\x{2193}"  => '&darr;',
+		"\x{2194}"  => '&harr;',
+		"\x{21b5}"  => '&crarr;',
+		"\x{21d0}"  => '&lArr;',
+		"\x{21d1}"  => '&uArr;',
+		"\x{21d2}"  => '&rArr;',
+		"\x{21d3}"  => '&dArr;',
+		"\x{21d4}"  => '&hArr;',
+		"\x{2200}"  => '&forall;',
+		"\x{2202}"  => '&part;',
+		"\x{2203}"  => '&exist;',
+		"\x{2205}"  => '&empty;',
+		"\x{2207}"  => '&nabla;',
+		"\x{2208}"  => '&isin;',
+		"\x{2209}"  => '&notin;',
+		"\x{220b}"  => '&ni;',
+		"\x{220f}"  => '&prod;',
+		"\x{2211}"  => '&sum;',
+		"\x{2212}"  => '&minus;',
+		"\x{2217}"  => '&lowast;',
+		"\x{221a}"  => '&radic;',
+		"\x{221d}"  => '&prop;',
+		"\x{221e}"  => '&infin;',
+		"\x{2220}"  => '&ang;',
+		"\x{2227}"  => '&and;',
+		"\x{2228}"  => '&or;',
+		"\x{2229}"  => '&cap;',
+		"\x{222a}"  => '&cup;',
+		"\x{222b}"  => '&int;',
+		"\x{2234}"  => '&there4;',
+		"\x{223c}"  => '&sim;',
+		"\x{2245}"  => '&cong;',
+		"\x{2248}"  => '&asymp;',
+		"\x{2260}"  => '&ne;',
+		"\x{2261}"  => '&equiv;',
+		"\x{2264}"  => '&le;',
+		"\x{2265}"  => '&ge;',
+		"\x{2282}"  => '&sub;',
+		"\x{2283}"  => '&sup;',
+		"\x{2284}"  => '&nsub;',
+		"\x{2286}"  => '&sube;',
+		"\x{2287}"  => '&supe;',
+		"\x{2295}"  => '&oplus;',
+		"\x{2297}"  => '&otimes;',
+		"\x{22a5}"  => '&perp;',
+		"\x{22c5}"  => '&sdot;',
+		"\x{2308}"  => '&lceil;',
+		"\x{2309}"  => '&rceil;',
+		"\x{230a}"  => '&lfloor;',
+		"\x{230b}"  => '&rfloor;',
+		"\x{2329}"  => '&lang;',
+		"\x{232a}"  => '&rang;',
+		"\x{25ca}"  => '&loz;',
+		"\x{2660}"  => '&spades;',
+		"\x{2663}"  => '&clubs;',
+		"\x{2665}"  => '&hearts;',
+		"\x{2666}"  => '&diams;',
+		);	
 }
 
 sub decode_entities ($@)
@@ -2520,6 +2766,11 @@ notations.
 There contain the mapping from all characters to the corresponding entities
 (and vice versa, respectively). These variables may be exported.
 
+Note that C<< %char2entity >> is a more conservative set of mappings, intended to
+be safe for serialising strings to HTML4, HTML5 and XHTML 1.x. And for
+hysterical raisins, C<< %entity2char >> does not include the leading ampersands,
+while C<< %char2entity >> does.
+
 =back
 
 =head1 BUGS
@@ -2541,7 +2792,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 Copyright (c) 1995-2006 by Gisle Aas.
 
-Copyright (c) 2011 by Toby Inkster.
+Copyright (c) 2012 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
@@ -2553,7 +2804,7 @@ and Opera Software ASA.
 
 Copyright (c) 2007-2011 by Wakaba <w@suika.fam.cx>.
 
-Copyright (c) 2009-2011 by Toby Inkster <tobyink@cpan.org>.
+Copyright (c) 2009-2012 by Toby Inkster <tobyink@cpan.org>.
 
 =head1 DISCLAIMER OF WARRANTIES
 
